@@ -20,7 +20,6 @@ The ``fedora-messaging`` `Click`_ CLI.
 .. _Click: http://click.pocoo.org/
 """
 
-
 import errno
 import importlib
 import logging
@@ -29,9 +28,11 @@ import os
 import sys
 
 import click
-import pkg_resources
+import requests
 from twisted.internet import asyncioreactor, error
 
+from fedora_messaging import message
+
 
 try:
     asyncioreactor.install()
@@ -42,11 +43,11 @@ except error.ReactorAlreadyInstalledError:
     if not isinstance(reactor, asyncioreactor.AsyncioSelectorReactor):
         raise
 
-from twisted.internet import reactor  # noqa: E402
-from twisted.python import log as legacy_twisted_log  # noqa: E402
+from twisted.internet import reactor
+from twisted.python import log as legacy_twisted_log
 
-from . import api, config, exceptions  # noqa: E402
-from .message import dumps, loads  # noqa: E402
+from . import api, config, exceptions
+from .message import dumps, loads
 
 
 _log = logging.getLogger(__name__)
@@ -91,6 +92,9 @@ _publish_exchange_help = (
     "The name of the exchange to publish to. Can contain ASCII letters, "
     "digits, hyphen, underscore, period, or colon."
 )
+_datagrepper_help = (
+    "The URL of the datagreeper instance to use, " "defaults to the production environment"
+)
 _limit_help = "The maximum number of messages to record."
 
 
@@ -109,7 +113,7 @@ def cli(conf):
         try:
             config.conf.load_config(config_path=conf)
         except exceptions.ConfigurationException as e:
-            raise click.exceptions.BadParameter(str(e))
+            raise click.exceptions.BadParameter(str(e)) from e
     twisted_observer = legacy_twisted_log.PythonLoggingObserver()
     twisted_observer.start()
     config.conf.setup_logging()
@@ -166,17 +170,11 @@ def _consume(exchange, queue_name, routing_key, callback, app_name):
         config.conf["client_properties"]["app"] = app_name
 
     try:
-        deferred_consumers = api.twisted_consume(
-            callback, bindings=bindings, queues=queues
-        )
+        deferred_consumers = api.twisted_consume(callback, bindings=bindings, queues=queues)
         deferred_consumers.addCallback(_consume_callback)
         deferred_consumers.addErrback(_consume_errback)
     except ValueError as e:
-        click_version = pkg_resources.get_distribution("click").parsed_version
-        if click_version < pkg_resources.parse_version("7.0"):
-            raise click.exceptions.BadOptionUsage(str(e))
-        else:
-            raise click.exceptions.BadOptionUsage("callback", str(e))
+        raise click.exceptions.BadOptionUsage("callback", str(e)) from e
 
     reactor.run()
     sys.exit(_exit_code)
@@ -198,13 +196,13 @@ def _callback_from_filesystem(callback_file):
     """
     try:
         file_path, callable_name = callback_file.strip().split(":")
-    except ValueError:
+    except ValueError as e:
         raise click.ClickException(
             "Unable to parse the '--callback-file' option; the "
             'expected format is "path/to/file.py:callable_object" where '
             '"callable_object" is the name of the function or class in the '
             "Python file"
-        )
+        ) from e
 
     try:
         file_namespace = {}
@@ -213,21 +211,19 @@ def _callback_from_filesystem(callback_file):
                 # Using "exec" is generally a Bad Idea (TM), so bandit is upset at
                 # us. In this case, it seems like a Good Idea (TM), but I might be
                 # wrong. Sorry.
-                exec(compile(fd.read(), file_path, "exec"), file_namespace)  # nosec
+                exec(compile(fd.read(), file_path, "exec"), file_namespace)  # noqa: S102
             except Exception as e:
                 raise click.ClickException(
                     f"The {file_path} file raised the following exception during execution: {e}"
-                )
+                ) from e
 
         if callable_name not in file_namespace:
-            err = "The '{}' object was not found in the '{}' file.".format(
-                callable_name, file_path
-            )
+            err = f"The '{callable_name}' object was not found in the '{file_path}' file."
             raise click.ClickException(err)
         else:
             return file_namespace[callable_name]
     except OSError as e:
-        raise click.ClickException(f"An IO error occurred: {e}")
+        raise click.ClickException(f"An IO error occurred: {e}") from e
 
 
 def _callback_from_python_path(callback):
@@ -252,19 +248,19 @@ def _callback_from_python_path(callback):
         )
     try:
         module, cls = callback_path.strip().split(":")
-    except ValueError:
+    except ValueError as e:
         raise click.ClickException(
-            "Unable to parse the callback path ({}); the "
+            f"Unable to parse the callback path ({callback_path}); the "
             'expected format is "my_package.module:'
-            'callable_object"'.format(callback_path)
-        )
+            'callable_object"'
+        ) from e
     try:
         module = importlib.import_module(module)
     except ImportError as e:
         provider = "--callback argument" if callback else "configuration file"
         raise click.ClickException(
             f"Failed to import the callback module ({e}) provided in the {provider}"
-        )
+        ) from e
 
     try:
         callback_object = getattr(module, cls)
@@ -272,7 +268,7 @@ def _callback_from_python_path(callback):
         raise click.ClickException(
             f"Unable to import {callback_path} ({e}); is the package installed? The python "
             'path should be in the format "my_package.module:callable_object"'
-        )
+        ) from e
     _log.info("Starting consumer with %s callback", callback_path)
     return callback_object
 
@@ -323,7 +319,7 @@ def _consume_callback(consumers):
     """
     for consumer in consumers:
 
-        def errback(failure):
+        def errback(failure, consumer=consumer):
             global _exit_code
             if failure.check(exceptions.HaltConsumer):
                 _exit_code = failure.value.exit_code
@@ -346,9 +342,7 @@ def _consume_callback(consumers):
                 )
             else:
                 _exit_code = 13
-                _log.error(
-                    "Unexpected error occurred in consumer %r: %r", consumer, failure
-                )
+                _log.error("Unexpected error occurred in consumer %r: %r", consumer, failure)
             try:
                 reactor.stop()
             except error.ReactorNotRunning:
@@ -376,7 +370,7 @@ def publish(exchange, file):
         try:
             messages = loads(msgs_json_str)
         except exceptions.ValidationError as e:
-            raise click.BadArgumentUsage(f"Unable to validate message: {e}")
+            raise click.BadArgumentUsage(f"Unable to validate message: {e}") from e
 
         for msg in messages:
             click.echo(f"Publishing message with topic {msg.topic}")
@@ -428,7 +422,7 @@ class Recorder:
             json_str = dumps(message)
         except exceptions.ValidationError as e:
             click.echo(f"Unable to save messages to file: {e}")
-            raise exceptions.HaltConsumer(exit_code=1, requeue=False)
+            raise exceptions.HaltConsumer(exit_code=1, requeue=False) from e
         else:
             self._file.write(json_str)
 
@@ -449,6 +443,72 @@ class Recorder:
 def record(exchange, queue_name, routing_key, app_name, limit, file):
     """Record messages from an AMQP queue to provided file."""
     messages_recorder = Recorder(limit, file)
-    _consume(
-        exchange, queue_name, routing_key, messages_recorder.collect_message, app_name
-    )
+    _consume(exchange, queue_name, routing_key, messages_recorder.collect_message, app_name)
+
+
+DEFAULT_DATAGREPPER_URL = "https://apps.fedoraproject.org/datagrepper"
+
+
+def _get_message(message_id, datagrepper_url):
+    """Fetch a message by ID from Datagreeper"""
+    url = f"{datagrepper_url}/v2/id?id={message_id}&is_raw=true"
+    try:
+        response = requests.get(url, timeout=5)
+        response.raise_for_status()
+        return response.json()
+    except requests.exceptions.RequestException as e:
+        raise click.ClickException(f"Failed to retrieve message from Datagrepper: {e}") from e
+
+
+@cli.command()
+@click.argument("message_id")
+@click.option(
+    "--datagrepper-url",
+    help=_datagrepper_help,
+    default=DEFAULT_DATAGREPPER_URL,
+    show_default=True,
+)
+def replay(message_id, datagrepper_url):
+    """Replay a message from Datagrepper by its message ID"""
+    message_data = _get_message(message_id, datagrepper_url)
+    if message_data:
+        # Disable the topic prefix, the loaded message already has everything.
+        config.conf["topic_prefix"] = ""
+        api.publish(message.load_message(message_data))
+        click.echo(f"Message with ID {message_id} has been successfully replayed.")
+
+
+@cli.command()
+@click.argument("message_id")
+@click.option("--callback-file", help=_callback_file_help)
+@click.option("--callback", help=_callback_help)
+@click.option(
+    "--datagrepper-url",
+    help=_datagrepper_help,
+    default=DEFAULT_DATAGREPPER_URL,
+    show_default=True,
+)
+def reconsume(
+    datagrepper_url: str,
+    callback: str,
+    callback_file: click.Path,
+    message_id: click.types.UUIDParameterType,
+):
+    """
+    Re-consume a message from Datagrepper.
+
+    This command will load a message from Datagrepper and then pass it
+    to the configured consumer. This can be useful when testing a new
+    consumer, or in the event that your consumer accidentally acknowledged
+    a message it failed to properly process.
+    """
+    if callback_file:
+        callback = _callback_from_filesystem(callback_file)
+    else:
+        callback = _callback_from_python_path(callback)
+    # It's not a public API, but hopefully we will not break ourselves.
+    callback_instance = api._check_callback(callback)
+    msg = _get_message(message_id, datagrepper_url)
+    config.conf["topic_prefix"] = ""
+    msg = message.load_message(msg)
+    callback_instance(msg)
