@@ -1,5 +1,7 @@
 """Exceptions raised by Fedora Messaging."""
 
+import jsonschema
+
 
 class BaseException(Exception):
     """The base class for all exceptions raised by fedora_messaging."""
@@ -31,8 +33,9 @@ class PermissionException(BaseException):
         return self.description
 
     def __repr__(self):
-        return "PermissionException(obj_type={}, description={}, reason={})".format(
-            self.obj_type, self.description, self.reason
+        return (
+            f"PermissionException(obj_type={self.obj_type}, description={self.description}, "
+            f"reason={self.reason})"
         )
 
 
@@ -48,13 +51,15 @@ class BadDeclaration(PermissionException):
     """
 
     def __str__(self):
-        return "Unable to declare the {} object ({}) because {}".format(
-            self.obj_type, self.description, self.reason
+        return (
+            f"Unable to declare the {self.obj_type} object ({self.description}) "
+            f"because {self.reason}"
         )
 
     def __repr__(self):
-        return "BadDeclaration(obj_type={}, description={}, reason={})".format(
-            self.obj_type, self.description, self.reason
+        return (
+            f"BadDeclaration(obj_type={self.obj_type}, description={self.description}, "
+            f"reason={self.reason})"
         )
 
 
@@ -179,3 +184,11 @@ class ValidationError(BaseException):
     and testing that you're trying to publish a message with a different
     format, and that you should either fix it or update the schema.
     """
+
+    @property
+    def summary(self):
+        """A short summary of the error."""
+        original_exception = self.args[0]
+        if isinstance(original_exception, jsonschema.exceptions.ValidationError):
+            return original_exception.message
+        return str(original_exception)
